@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -278,7 +278,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
       number_planes++;
     number_pixels=(MagickSizeType) image->columns*image->rows;
     if ((GetBlobSize(image) == 0) || ((((MagickSizeType) number_pixels*
-         number_planes*bits_per_pixel/8)/GetBlobSize(image)) > 254.0))
+         number_planes*bits_per_pixel/8)/GetBlobSize(image)) > 254))
       ThrowRLEException(CorruptImageError,"InsufficientImageDataInFile")
     if (((MagickSizeType) number_colormaps*map_length) > GetBlobSize(image))
       ThrowRLEException(CorruptImageError,"InsufficientImageDataInFile")
@@ -320,8 +320,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
             comment=(char *) AcquireQuantumMemory(length,sizeof(*comment));
             if (comment == (char *) NULL)
               ThrowRLEException(ResourceLimitError,"MemoryAllocationFailed");
-            (void) ReadBlob(image,length-1,(unsigned char *) comment);
-            if (count != (length-1))
+            count=ReadBlob(image,length-1,(unsigned char *) comment);
+            if (count != (ssize_t) (length-1))
               {
                 comment=DestroyString(comment);
                 ThrowRLEException(CorruptImageError,"UnexpectedEndOfFile");
@@ -704,8 +704,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            image=DestroyImageList(image);
-            return((Image *) NULL);
+            status=MagickFalse;
+            break;
           }
         image=SyncNextImageInList(image);
         status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
@@ -715,6 +715,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
   } while ((count != 0) && (memcmp(magick,"\122\314",2) == 0));
   (void) CloseBlob(image);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
