@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -102,7 +102,8 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   char
     **filelist,
-    *label;
+    *label,
+    **list;
 
   Image
     *image,
@@ -140,12 +141,15 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
-  filelist=(char **) AcquireMagickMemory(sizeof(*filelist));
-  if (filelist == (char **) NULL)
+  list=(char **) AcquireMagickMemory(sizeof(*filelist));
+  if (list == (char **) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  filelist[0]=ConstantString(image_info->filename);
+  list[0]=ConstantString(image_info->filename);
+  filelist=list;
   number_files=1;
   status=ExpandFilenames(&number_files,&filelist);
+  list[0]=DestroyString(list[0]);
+  list=(char **) RelinquishMagickMemory(list);
   if ((status == MagickFalse) || (number_files == 0))
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   image=DestroyImage(image);
@@ -172,8 +176,11 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (next_image == (Image *) NULL)
       break;
     label=InterpretImageProperties(image_info,next_image,DefaultTileLabel);
-    (void) SetImageProperty(next_image,"label",label);
-    label=DestroyString(label);
+    if (label != (char *) NULL)
+      {
+        (void) SetImageProperty(next_image,"label",label);
+        label=DestroyString(label);
+      }
     if (image_info->debug != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "geometry: %.20gx%.20g",(double) next_image->columns,(double)
