@@ -17,13 +17,13 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -53,6 +53,7 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/version.h"
 #include "magick/xml-tree.h"
 #include "magick/xml-tree-private.h"
 
@@ -60,29 +61,6 @@
   Define declarations.
 */
 #define ConfigureFilename  "configure.xml"
-
-#ifdef _OPENMP
-#define MAGICKCORE_FEATURE_OPENMP_STR "OpenMP "
-#else
-#define MAGICKCORE_FEATURE_OPENMP_STR ""
-#endif
-#ifdef _OPENCL
-#define MAGICKCORE_FEATURE_OPENCL_STR "OpenCL "
-#else
-#define MAGICKCORE_FEATURE_OPENCL_STR ""
-#endif
-#ifdef MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
-#define MAGICKCORE_FEATURE_ZERO_CONFIGURATION_STR "Zero-Configuration "
-#else
-#define MAGICKCORE_FEATURE_ZERO_CONFIGURATION_STR ""
-#endif
-#ifdef HDRI_SUPPORT
-#define MAGICKCORE_FEATURE_HDRI_STR "HDRI"
-#else
-#define MAGICKCORE_FEATURE_HDRI_STR ""
-#endif
-
-#define MAGICKCORE_FEATURES_STR MAGICKCORE_FEATURE_OPENMP_STR MAGICKCORE_FEATURE_OPENCL_STR MAGICKCORE_FEATURE_ZERO_CONFIGURATION_STR MAGICKCORE_FEATURE_HDRI_STR
 
 /*
   Typedef declarations.
@@ -97,14 +75,6 @@ typedef struct _ConfigureMapInfo
 /*
   Static declarations.
 */
-static const ConfigureMapInfo
-  ConfigureMap[] =
-  {
-    { "NAME", "ImageMagick" },
-    { "QuantumDepth", MAGICKCORE_STRING_XQUOTE(MAGICKCORE_QUANTUM_DEPTH) },
-    { "FEATURES", MAGICKCORE_FEATURES_STR }
-  };
-
 static LinkedListInfo
   *configure_cache = (LinkedListInfo *) NULL;
 
@@ -145,25 +115,36 @@ static MagickBooleanType
 %    o exception: return any errors or warnings in this structure.
 %
 */
+static inline void AddConfigureKey(LinkedListInfo *cache,const char *path,
+  const char *name,const char *value)
+{
+  ConfigureInfo
+    *configure_info;
+
+  configure_info=(ConfigureInfo *) AcquireMagickMemory(sizeof(*configure_info));
+  if (configure_info == (ConfigureInfo *) NULL)
+    return;
+  (void) memset(configure_info,0,sizeof(*configure_info));
+  configure_info->path=(char *) path;
+  configure_info->name=(char *) name;
+  configure_info->value=(char *) value;
+  configure_info->exempt=MagickTrue;
+  configure_info->signature=MagickCoreSignature;
+  (void) AppendValueToLinkedList(cache,configure_info);
+}
+
 static LinkedListInfo *AcquireConfigureCache(const char *filename,
   ExceptionInfo *exception)
 {
   LinkedListInfo
     *cache;
 
-  MagickStatusType
-    status;
-
-  register ssize_t
-    i;
-
   /*
     Load external configure map.
   */
   cache=NewLinkedList(0);
   if (cache == (LinkedListInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  status=MagickTrue;
 #if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
   {
     const StringInfo
@@ -172,49 +153,33 @@ static LinkedListInfo *AcquireConfigureCache(const char *filename,
     LinkedListInfo
       *options;
 
+    MagickBooleanType
+      status;
+
     options=GetConfigureOptions(filename,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
     while (option != (const StringInfo *) NULL)
     {
-      status&=LoadConfigureCache(cache,(const char *)
+      status=LoadConfigureCache(cache,(const char *)
         GetStringInfoDatum(option),GetStringInfoPath(option),0,exception);
+      if (status == MagickTrue)
+        break;
       option=(const StringInfo *) GetNextValueInLinkedList(options);
     }
     options=DestroyConfigureOptions(options);
   }
 #endif
   /*
-    Load built-in configure map.
+    Load built-in configure.
   */
-  for (i=0; i < (ssize_t) (sizeof(ConfigureMap)/sizeof(*ConfigureMap)); i++)
-  {
-    ConfigureInfo
-      *configure_info;
-
-    register const ConfigureMapInfo
-      *p;
-
-    p=ConfigureMap+i;
-    configure_info=(ConfigureInfo *) AcquireMagickMemory(
-      sizeof(*configure_info));
-    if (configure_info == (ConfigureInfo *) NULL)
-      {
-        (void) ThrowMagickException(exception,GetMagickModule(),
-          ResourceLimitError,"MemoryAllocationFailed","`%s'",p->name);
-        continue;
-      }
-    (void) memset(configure_info,0,sizeof(*configure_info));
-    configure_info->path=(char *) "[built-in]";
-    configure_info->name=(char *) p->name;
-    configure_info->value=(char *) p->value;
-    configure_info->exempt=MagickTrue;
-    configure_info->signature=MagickCoreSignature;
-    status&=AppendValueToLinkedList(cache,configure_info);
-    if (status == MagickFalse)
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",
-        configure_info->name);
-  }
+  AddConfigureKey(cache,"[built-in]","NAME","ImageMagick");
+  /*
+    Load runtime configuration.
+  */
+  AddConfigureKey(cache,"[built-in]","QuantumDepth",GetMagickQuantumDepth(
+    (size_t *)NULL));
+  AddConfigureKey(cache,"[built-in]","FEATURES",GetMagickFeatures());
+  AddConfigureKey(cache,"[built-in]","DELEGATES",GetMagickDelegates());
   return(cache);
 }
 
@@ -889,12 +854,14 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
       *home;
 
     home=GetEnvironmentValue("XDG_CONFIG_HOME");
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__MINGW32__)
     if (home == (char *) NULL)
       home=GetEnvironmentValue("LOCALAPPDATA");
     if (home == (char *) NULL)
       home=GetEnvironmentValue("APPDATA");
     if (home == (char *) NULL)
       home=GetEnvironmentValue("USERPROFILE");
+#endif
     if (home != (char *) NULL)
       {
         /*
